#!./tclkit
# install.tcl

set pfmVersion {2.0.6}
package require msgcat 1.4.2
namespace import ::msgcat::mc
global env

proc getInput {} {
    set input [gets stdin]
    puts stdout [string repeat - 60]
    return $input
}


set currentDir [file normalize .]
::msgcat::mcload [file join $currentDir installer msgs]

puts stdout [mc installerWelcome]
getInput

set uninstall {}
set stop 0
while {!$stop} {
    set useTclkit 1
    puts stdout [mc useTclkit]
    if {[getInput] in [mc answerNo]} then {
        set useTclkit 0
        if {[catch {exec which tclsh} tclsh]} then {
            set tclsh "/usr/bin/tclsh"
        }
        puts stdout [mc enterTclshPath $tclsh]
        set answer [getInput]
        if {$answer ne {}} then {
            set tclsh $answer
        }
        if {[catch {
            exec $tclsh \
                [file join $currentDir installer check_packages.tcl]
            } versions]} then {
            puts stdout [mc checkPackagesFailed $versions]
        } else {
            puts stdout [mc packagesFound]
            set required [list 8.5 8.5 3.4 1.4.2 [mc optional] [mc optional]]
            puts stdout [format {%20s %10s %10s} [mc package] \
                [mc available] [mc required]]
            set idx 0
            foreach package $versions {
                puts stdout [format {%20s %10s %10s} [lindex $package 0] \
                    [lindex $package 1] [lindex $required $idx]]
                incr idx
            }
            puts stdout [mc confident]
            if {[getInput] ni [mc answerNo]} then {
                set stop 1
            }
        }
    } else {
        set useTclkit 1
        set stop 1
    }
}

if {$tcl_platform(user) eq {root}} then {
    set systemInstall 1
    set installDir "/usr/share/pfm-$pfmVersion"
    set launchDir {/usr/bin}
} else {
    set systemInstall 0
    set installDir [file join [file normalize ~] pfm-$pfmVersion]
    set launchDir [file join [file normalize ~] bin]
}
puts stdout [mc getInstallDir $installDir]
set answer [getInput]
if {$answer ne {}} then {
    set installDir $answer
}
puts stdout [mc getLaunchDir $launchDir]
set answer [getInput]
if {$answer ne {}} then {
    set launchDir $answer
}

if {$useTclkit} then {
    set usepgintcl 1
    append uninstall "#![file join $installDir tclkit]\n"
} else {
    append uninstall "#!${tclsh}\n"
    puts stdout [mc usePgintcl]
    if {[getInput] ni [mc answerNo]} then {
        set usepgintcl 1
    } else {
        set usepgintcl 0
    }
}
set fileList [list \
arrow-end.xbm \
arrow-home.xbm \
arrow-left.xbm \
arrow-right.xbm \
cat.tcl \
CHANGE_LOG.txt \
config.tcl \
convert_from_1.0.4.sql \
convert_from_1.1.0.sql \
convert_from_1.2.0.sql \
database.tcl \
forms.tcl \
install_pfm.sql \
misc.tcl \
options.tcl \
pfm.conf \
pfm-icon.png \
pfm.tcl \
postgresql.tcl \
README.txt \
report.tcl \
sql.tcl \
doc \
examples \
license \
msgs]

if {[file exists $installDir] && [file isdirectory $installDir]} then {
    puts stdout [mc dirAlreadyExists $installDir]
    if {[getInput] in [mc answerYes]} then {
        file delete -force $installDir
    } else {
        puts stdout [mc installationAborted]
        exit
    }
}

if {[catch {file mkdir $installDir} errMsg]} then {
    puts stdout [mc couldNotMakeInstallDir $installDir $errMsg]
    puts stdout [mc installationAborted]
    exit
}

foreach file $fileList {
    puts stdout "cp $file $installDir"
    file copy $file $installDir
}

if {$useTclkit} then {
    puts stdout "cp tclkit $installDir"
    # file copy tclkit $installDir
    # could not use the above instruction because tclkit sees the
    # virtual file system within tclkit and unwraps it
    if {[catch {
            exec cp tclkit $installDir
            } errMsg]} then {
        puts stdout [mc installTclkitFailed $errMsg]
    }
    puts stdout "cp tclkit-README.txt $installDir"
    file copy tclkit-README.txt $installDir
    set launchCmd \
        "[file join $installDir tclkit] [file join $installDir pfm.tcl]"
} else {
    set launchCmd "$tclsh [file join $installDir pfm.tcl]"
}

if {$usepgintcl} then {
    puts "cp pgintcl-3.2.1 $installDir"
    file copy pgintcl-3.2.1 $installDir
    file link -symbolic [file join $installDir pgin.tcl] \
        [file join $installDir pgintcl-3.2.1 pgin.tcl]
}

# install program launcher in $launchDir
set launchChan [open [file join $launchDir pfm] w]
puts $launchChan {#!/bin/sh}
puts $launchChan $launchCmd
puts $launchChan exit
chan close $launchChan
file attributes [file join $launchDir pfm] -permissions a+x
append uninstall "file delete [file join $launchDir pfm]\n"
puts stdout [mc launchScriptWritten [file join $launchDir pfm]]

# make a desktop file in installDir
set desktopChan [open pfm-postgresforms.template r]
set desktop [chan read $desktopChan]
chan close $desktopChan
set desktopChan [open [file join $installDir pfm-postgresforms.desktop] w]
chan puts $desktopChan [format $desktop $launchCmd $installDir]
chan close $desktopChan

# check for installation of xdg-utils
set checkForXdg 1
while {$checkForXdg} {
    if {[catch {exec which xdg-desktop-menu} xdgPath]} then {
        puts stdout [mc whichXdg $xdgPath]
        puts stdout [mc checkXdg]
        if {[getInput] in [mc answerNo]} then {
            set checkForXdg 0
        }
    } else {
        puts stdout [mc whichXdg $xdgPath]
        set checkForXdg 0
    }
}

# install desktop menu entry
puts stdout [mc installMenu]
set menu 0
if {[getInput] ni [mc answerNo]} then {
    puts stdout \
        "xdg-desktop-menu install [file join $installDir pfm-postgresforms.desktop]"
    if {![catch {
        exec xdg-desktop-menu install \
            [file join $installDir pfm-postgresforms.desktop]
        } result]} then {
            set menu 1
        append uninstall \
        "exec xdg-desktop-menu uninstall [file join $installDir pfm-postgresforms.desktop]\n"
    }
    puts stdout $result
}

# install desktop icon
set icon 0
if {!$systemInstall} then {
    puts stdout [mc installIcon]
    if {[getInput] ni [mc answerNo]} then {
        puts stdout \
            "xdg-desktop-icon install [file join $installDir pfm-postgresforms.desktop]"
        if {![catch {
            exec xdg-desktop-icon install \
                [file join $installDir pfm-postgresforms.desktop]
            } result]} then {
            set icon 1
            append uninstall "exec xdg-desktop-icon uninstall [file join $installDir pfm-postgresforms.desktop]\n"
        }
        puts stdout $result
    }
}

# install uninstall script
append uninstall "file delete -force $installDir\nexit\n"
set uninstChan [open [file join $installDir uninstall.tcl] w]
puts $uninstChan $uninstall
close $uninstChan
file attribute [file join $installDir uninstall.tcl] -permissions a+x

# Change ownership of installDir and of all files in it
if {$systemInstall} then {
    set owner [file attributes [file join $installDir pfm.tcl] -owner]
    if {$owner ne {root}} then {
        puts stdout [mc changeOwnership $owner]
        if {[getInput] ni [mc answerNo]} then {
            puts stdout "chown --recursive root:root $installDir"
            if {[catch {exec chown --recursive root:root $installDir} result]} then {
                puts stdout $result
                puts stdout [mc chownFailed]
            }
        }
    }
}

puts stdout [mc installationComplete $installDir]

puts stdout [mc howToStart]

if {$launchDir in [split $env(PATH) {:}]} then {
    puts stdout [mc startWhenInPath]
} else {
    puts stdout [mc startWhenNotInPath $launchDir]
}

if {$menu} then {
    puts stdout [mc startFromMenu]
}

if {$icon} then {
    puts stdout [mc startFromDesktopIcon]
}

if {!$menu && !$icon} then {
    puts stdout [mc desktopIconInInstallDir $installDir]
}

exit
